// hdshow.cpp

// Hello, DirectShow!

#include <control.h>
#include <iostream.h>
#include <strmif.h>
#include <uuids.h>

void main (int argc, char *argv [])
{
   if (argc != 2)
   {
       cerr << "usage: hdshow filespec\n";
       return;
   }

   if (FAILED (CoInitialize (NULL)))
   {
       cerr << "hdshow: CoInitialize() failure\n";
       return;
   }

   IGraphBuilder *pGraph;
   HRESULT hr = CoCreateInstance (CLSID_FilterGraph, NULL,
                                  CLSCTX_INPROC_SERVER, IID_IGraphBuilder,
                                  (LPVOID *) &pGraph);
   if (FAILED (hr))
   {
       cerr << "hdshow: CoCreateInstance() failure\n";
       CoUninitialize ();
       return;
   }

   IMediaControl *pControl;
   hr = pGraph->QueryInterface (IID_IMediaControl, (LPVOID *) &pControl);
   if (FAILED (hr))
   {
       cerr << "hdshow: unable to obtain IMediaControl interface\n";
       pGraph->Release ();
       CoUninitialize ();
       return;
   }

   IMediaEvent *pEvent;
   hr = pGraph->QueryInterface (IID_IMediaEvent, (LPVOID *) &pEvent);
   if (FAILED (hr))
   {
       cerr << "hdshow: unable to obtain IMediaEvent interface\n";
       pControl->Release ();
       pGraph->Release ();
       CoUninitialize ();
       return;
   }

   WCHAR wPath [MAX_PATH];
   MultiByteToWideChar (CP_ACP, 0, argv [1], -1, wPath, MAX_PATH);

   hr = pGraph->RenderFile (wPath, NULL);
   if (SUCCEEDED (hr))
   {
       hr = pControl->Run ();
       if (SUCCEEDED (hr))
       {
           long evCode;
           pEvent->WaitForCompletion (INFINITE, &evCode);
       }
       else
           cerr << "hdshow: unable to run " << argv [1] << "\n";
   }
   else
       cerr << "hdshow: unable to render " << argv [1] << "\n";

   pEvent->Release ();
   pControl->Release ();
   pGraph->Release ();
   CoUninitialize ();
}
