// mp1.cpp

// Media Player #1

#include <control.h>
#include <io.h>       // Allow access to MAXPATH, MAXFILE, and MAXEXT constants.
#include <iostream.h>
#include <strmif.h>
#include <uuids.h>

#include "mp1.h"

#define OAFALSE 0
#define OATRUE -1

#pragma resource "mp1.res"

char g_szAppName [] = "mp1";

IGraphBuilder *g_pGraph;

IMediaControl *g_pControl;

IVideoWindow *g_pVidWin;

void Cleanup (void)
{
   if (g_pControl != NULL)
       g_pControl->Stop ();

   if (g_pVidWin != NULL)
   {
       g_pVidWin->put_Visible (OAFALSE); // Hide video renderer's window to 
                                         // prevent momentary video image
                                         // flicker.
       g_pVidWin->put_Owner (NULL);
       g_pVidWin->Release ();
       g_pVidWin = NULL;
   }

   if (g_pControl != NULL)
   {
       g_pControl->Release ();
       g_pControl = NULL;
   }

   if (g_pGraph != NULL)
   {
       g_pGraph->Release ();
       g_pGraph = NULL;
   }
}

void PlayFile (HWND hWnd, LPSTR szMediaPath)
{
   Cleanup ();

   HRESULT hr = CoCreateInstance (CLSID_FilterGraph, NULL,
                                  CLSCTX_INPROC_SERVER, IID_IGraphBuilder,
                                  (LPVOID *) &g_pGraph);
   if (FAILED (hr))
   {
       MessageBox (hWnd, "CoCreateInstance() failure", g_szAppName, MB_OK);
       return;
   }

   hr = g_pGraph->QueryInterface (IID_IMediaControl, (LPVOID *) &g_pControl);
   if (FAILED (hr))
   {
       MessageBox (hWnd, "unable to obtain IMediaControl interface",
                   g_szAppName, MB_OK);
       return;
   }

   hr = g_pGraph->QueryInterface (IID_IVideoWindow, (LPVOID *) &g_pVidWin);
   if (FAILED (hr))
   {
       MessageBox (hWnd, "unable to obtain IVideoWindow interface",
                   g_szAppName, MB_OK);
       return;
   }

   WCHAR wPath [MAXPATH];
   MultiByteToWideChar (CP_ACP, 0, szMediaPath, -1, wPath, MAXPATH);

   hr = g_pGraph->RenderFile (wPath, NULL);
   if (SUCCEEDED (hr))
   {
       g_pVidWin->put_Owner ((OAHWND) hWnd);
       g_pVidWin->put_WindowStyle (WS_CHILD | WS_CLIPSIBLINGS);
       g_pVidWin->put_Visible (OATRUE);

       SendMessage (hWnd, WM_SIZE, 0, 0);

       hr = g_pControl->Run ();
       if (FAILED (hr))
           MessageBox (hWnd, "unable to run media", g_szAppName, MB_OK);
   }
   else
       MessageBox (hWnd, "unable to render media", g_szAppName, MB_OK);
}

LRESULT CALLBACK WndProc (HWND hWnd, int iMsg, WPARAM wParam, LPARAM lParam)
{
   LPSTR lpstrFilter = "Media files (*.mpg, *.avi, *.wmv, *.mp3, *.wma)\0"
                       "*.mpg;*.avi;*.wmv;*.mp3;*.wma\0"
                       "All files (*.*)\0"
                       "*.*\0"
                       "\0";

   switch (iMsg)
   {
      case WM_COMMAND:
           switch (LOWORD (wParam))
           {
              case IDM_FILE_EXIT:
                   SendMessage (hWnd, WM_CLOSE, 0, 0);
                   return 0;

              case IDM_FILE_OPEN:
                   char szFilename [MAXPATH];
                   szFilename [0] = '\0';

                   char szTitlename [MAXPATH];
                   szTitlename [0] = '\0';

                   OPENFILENAME ofn;
                   ofn.lStructSize = sizeof (OPENFILENAME);
                   ofn.hwndOwner = hWnd;
                   ofn.hInstance = NULL;
                   ofn.lpstrFilter = lpstrFilter;
                   ofn.lpstrCustomFilter = NULL;
                   ofn.nMaxCustFilter = 0;
                   ofn.nFilterIndex = 1;
                   ofn.lpstrFile = szFilename;
                   ofn.nMaxFile = MAXPATH;
                   ofn.lpstrFileTitle = szTitlename;
                   ofn.nMaxFileTitle = MAXFILE+MAXEXT;
                   ofn.lpstrInitialDir = NULL;
                   ofn.lpstrTitle = NULL;
                   ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
                   ofn.nFileOffset = 0;
                   ofn.nFileExtension = 0;
                   ofn.lpstrDefExt = NULL;
                   ofn.lCustData = 0;
                   ofn.lpfnHook = NULL;
                   ofn.lpTemplateName = NULL;
                   if (!GetOpenFileName (&ofn))
                       return 0;

                   PlayFile (hWnd, szTitlename);
                   return 0;
           }
           break;

      case WM_DESTROY:
           Cleanup ();
           PostQuitMessage (0);
           return 0;

      case WM_MOVE:
           if (g_pVidWin != NULL)
               g_pVidWin->NotifyOwnerMessage ((OAHWND) hWnd, iMsg, wParam,
                                              lParam);
           return 0;
          
      case WM_SIZE:
           if (g_pVidWin != NULL)
           {
               RECT rc;
               GetClientRect (hWnd, &rc);
               g_pVidWin->SetWindowPosition (0, 0, rc.right, rc.bottom);
           }
           return 0;
   }

   return DefWindowProc (hWnd, iMsg, wParam, lParam);
}

#pragma argsused
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hInstancePrev,
                    LPSTR lpszCmdLine, int iCmdShow)
{
   if (FAILED (CoInitialize (NULL)))
   {
       cerr << "CoInitialize() failure\n";
       return -1;
   }

   WNDCLASSEX wc;
   wc.cbSize = sizeof (WNDCLASSEX);
   wc.style = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc = (WNDPROC) WndProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = 0;
   wc.hInstance = hInstance;
   wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
   wc.hCursor = LoadCursor (NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
   wc.lpszMenuName = g_szAppName;
   wc.lpszClassName = g_szAppName;
   wc.hIconSm = LoadIcon (NULL, IDI_APPLICATION);

   RegisterClassEx (&wc);

   HWND hWnd = CreateWindowEx (0, g_szAppName, "Media Player #1",
                               WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
                               CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                               CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

   ShowWindow (hWnd, iCmdShow);
   UpdateWindow (hWnd);

   MSG msg;
   while (GetMessage (&msg, NULL, 0, 0 ))
   {
      TranslateMessage (&msg);
      DispatchMessage (&msg);
   }

   CoUninitialize ();

   return msg.wParam;
}
