// mp2.cpp

// Media Player #2

#include <control.h>
#include <evcode.h>   // EC_COMPLETE/EC_USERABORT/EC_ERRORABORT definitions
#include <io.h>
#include <iostream.h>
#include <strmif.h>
#include <uuids.h>

#include <commctrl.h> // toolbar and status bar constants

#include "mp2.h"

#define DELAY 100

#define OAFALSE 0
#define OATRUE -1

#define TBSTYLE_FLAT 0x0800

#define WM_GRAPHNOTIFY WM_APP+1

#pragma resource "mp2.res"

char g_szAppName [] = "mp2";

HWND g_hWndStatusBar;
HWND g_hWndToolBar;

IGraphBuilder *g_pGraph;

IMediaControl *g_pControl;

IMediaEventEx *g_pEvent;

IMediaPosition *g_pPosition;

IVideoWindow *g_pVidWin;

void DSCleanup (void)
{
   if (g_pControl != NULL)
       g_pControl->Stop ();

   if (g_pEvent != NULL)
   {
       g_pEvent->SetNotifyWindow (NULL, 0, 0);
       g_pEvent->Release ();
       g_pEvent = NULL;
   }

   if (g_pPosition != NULL)
   {
       g_pPosition->Release ();
       g_pPosition = NULL;
   }

   if (g_pVidWin != NULL)
   {
       g_pVidWin->put_Visible (OAFALSE); // Hide video renderer's window to 
                                         // prevent momentary video image
                                         // flicker.
       g_pVidWin->put_Owner (NULL);
       g_pVidWin->Release ();
       g_pVidWin = NULL;
   }

   if (g_pControl != NULL)
   {
       g_pControl->Release ();
       g_pControl = NULL;
   }

   if (g_pGraph != NULL)
   {
       g_pGraph->Release ();
       g_pGraph = NULL;
   }
}

BOOL DSSetup (HWND hWnd, LPSTR szMediaPath)
{
   HRESULT hr = CoCreateInstance (CLSID_FilterGraph, NULL,
                                  CLSCTX_INPROC_SERVER, IID_IGraphBuilder,
                                  (LPVOID *) &g_pGraph);
   if (FAILED (hr))
   {
       MessageBox (hWnd, "CoCreateInstance() failure", g_szAppName, MB_OK);
       return FALSE;
   }

   hr = g_pGraph->QueryInterface (IID_IMediaControl, (LPVOID *) &g_pControl);
   if (FAILED (hr))
   {
       MessageBox (hWnd, "unable to obtain IMediaControl interface",
                   g_szAppName, MB_OK);
       return FALSE;
   }

   hr = g_pGraph->QueryInterface (IID_IVideoWindow, (LPVOID *) &g_pVidWin);
   if (FAILED (hr))
   {
       MessageBox (hWnd, "unable to obtain IVideoWindow interface",
                   g_szAppName, MB_OK);
       return FALSE;
   }

   hr = g_pGraph->QueryInterface (IID_IMediaPosition, (LPVOID *) &g_pPosition);
   if (FAILED (hr))
   {
       MessageBox (hWnd, "unable to obtain IMediaPosition interface",
                   g_szAppName, MB_OK);
       return FALSE;
   }

   hr = g_pGraph->QueryInterface (IID_IMediaEventEx, (LPVOID *) &g_pEvent);
   if (FAILED (hr))
   {
       MessageBox (hWnd, "unable to obtain IMediaEventEx interface",
                   g_szAppName, MB_OK);
       return FALSE;
   }

   g_pEvent->SetNotifyWindow ((OAHWND) hWnd, WM_GRAPHNOTIFY, 0);

   WCHAR wPath [MAXPATH];
   MultiByteToWideChar (CP_ACP, 0, szMediaPath, -1, wPath, MAXPATH);

   hr = g_pGraph->RenderFile (wPath, NULL);
   if (SUCCEEDED (hr))
   {
       g_pVidWin->put_Owner ((OAHWND) hWnd);
       g_pVidWin->put_WindowStyle (WS_CHILD);
       g_pVidWin->put_Visible (OATRUE);

       RECT rcWin;
       GetClientRect (hWnd, &rcWin);
       WORD nWidth = rcWin.right-rcWin.left;
       WORD nHeight = rcWin.bottom-rcWin.top;
       SendMessage (hWnd, WM_SIZE, 0, MAKELONG (nWidth, nHeight));
   }
   else
   {
       MessageBox (hWnd, "unable to render media", g_szAppName, MB_OK);
       return FALSE;
   }

   return TRUE;
}

void UpdateStatusBar (void)
{
   REFTIME _seconds;
   if (FAILED (g_pPosition->get_CurrentPosition (&_seconds)))
       return;

   int seconds = (int) _seconds;
   int hours = seconds/3600;
   int minutes = (seconds-(hours*3600))/60;
   seconds = seconds-(hours*3600+minutes*60);

   char buffer [100];
   wsprintf (buffer, " %02d:%02d:%02d", hours, minutes, seconds);
   SendMessage (g_hWndStatusBar, SB_SETTEXT, 1, (LPARAM) (LPSTR) buffer);

   if (FAILED (g_pPosition->get_Duration (&_seconds)))
       return;

   seconds = (int) _seconds;
   hours = seconds/3600;
   minutes = (seconds-(hours*3600))/60;
   seconds = seconds-(hours*3600+minutes*60);

   wsprintf (buffer, " %02d:%02d:%02d", hours, minutes, seconds);
   SendMessage (g_hWndStatusBar, SB_SETTEXT, 2, (LPARAM) (LPSTR) buffer);
}

LRESULT CALLBACK WndProc (HWND hWnd, int iMsg, WPARAM wParam, LPARAM lParam)
{
   int aiPartWidths [3];

   LPSTR lpstrFilter = "Media files (*.mpg, *.avi, *.wmv, *.mp3, *.wma)\0"
                       "*.mpg;*.avi;*.wmv;*.mp3;*.wma\0"
                       "All files (*.*)\0"
                       "*.*\0"
                       "\0";

   switch (iMsg)
   {
      case WM_COMMAND:
           switch (LOWORD (wParam))
           {
              case IDM_FILE_EXIT:
                   SendMessage (hWnd, WM_CLOSE, 0, 0);
                   return 0;

              case IDM_FILE_OPEN:
                   char szFilename [MAXPATH];

                   if (lParam)
                       lstrcpy (szFilename, (LPSTR) lParam);
                   else
                   {
                       szFilename [0] = '\0';

                       OPENFILENAME ofn;
                       ofn.lStructSize = sizeof (OPENFILENAME);
                       ofn.hwndOwner = hWnd;
                       ofn.hInstance = NULL;
                       ofn.lpstrFilter = lpstrFilter;
                       ofn.lpstrCustomFilter = NULL;
                       ofn.nMaxCustFilter = 0;
                       ofn.nFilterIndex = 1;
                       ofn.lpstrFile = szFilename;
                       ofn.nMaxFile = MAXPATH;
                       ofn.lpstrFileTitle = NULL;
                       ofn.nMaxFileTitle = MAXFILE+MAXEXT;
                       ofn.lpstrInitialDir = NULL;
                       ofn.lpstrTitle = NULL;
                       ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
                       ofn.nFileOffset = 0;
                       ofn.nFileExtension = 0;
                       ofn.lpstrDefExt = NULL;
                       ofn.lCustData = 0;
                       ofn.lpfnHook = NULL;
                       ofn.lpTemplateName = NULL;
                       if (!GetOpenFileName (&ofn))
                           return 0;
                   }

                   DSCleanup ();

                   SendMessage (g_hWndStatusBar, SB_SETTEXT, 1, (LPARAM)
                                (LPSTR) " 00:00:00");
                   SendMessage (g_hWndStatusBar, SB_SETTEXT, 2, (LPARAM)
                                (LPSTR) " 00:00:00");

                   // Disable Play, Pause, and Stop buttons.

                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PLAY, 0);
                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PAUSE, 0);
                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_STOP, 0);

                   char szBuffer [MAXPATH*2];

                   if (DSSetup (hWnd, szFilename))
                   {
                       // Enable Play button.

                       SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PLAY,
                                    MAKELONG (TBSTATE_ENABLED, 0));

                       wsprintf (szBuffer, "%s - Media Player #2", szFilename);
                   }
                   else
                   {
                       DSCleanup ();

                       wsprintf (szBuffer, "Media Player #2", szFilename);
                       
                   }
                   SendMessage (hWnd, WM_SETTEXT, 0, (LPARAM) (LPSTR) szBuffer);
                   return 0;

              case IDM_PAUSE:
                   // Disable Pause and Stop buttons.

                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PAUSE, 0);
                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_STOP, 0);

                   // Enable Play button.

                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PLAY,
                                MAKELONG (TBSTATE_ENABLED, 0));

                   if (SUCCEEDED (g_pControl->Pause ()))
                       SendMessage (g_hWndStatusBar, SB_SETTEXT, 0, (LPARAM)
                                    (LPSTR) " Paused");

                   return 0;

              case IDM_PLAY:
                   // Disable Play button.

                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PLAY, 0);

                   // Enable Pause and Stop buttons.

                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PAUSE,
                                MAKELONG (TBSTATE_ENABLED, 0));
                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_STOP,
                                MAKELONG (TBSTATE_ENABLED, 0));

                   if (SUCCEEDED (g_pControl->Run ()))
                       SendMessage (g_hWndStatusBar, SB_SETTEXT, 0, (LPARAM)
                                    (LPSTR) " Running");
                   return 0;

              case IDM_STOP:
                   // Disable Pause and Stop buttons.

                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PAUSE, 0);
                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_STOP, 0);

                   // Enable Play button.

                   SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PLAY,
                                MAKELONG (TBSTATE_ENABLED, 0));

                   if (SUCCEEDED (g_pControl->Stop ()))
                   {
                       SendMessage (g_hWndStatusBar, SB_SETTEXT, 0, (LPARAM)
                                    (LPSTR) " Stopped");
                       g_pPosition->put_CurrentPosition (0);
                   }
                   return 0;
           }
           break;

      case WM_CREATE:
      {
           TBBUTTON tbb [3];

           tbb [0].iBitmap = 0;
           tbb [0].idCommand = IDM_PLAY;
           tbb [0].fsState = 0;
           tbb [0].fsStyle = TBSTYLE_BUTTON;
           tbb [0].dwData = 0;
           tbb [0].iString = 0;

           tbb [1].iBitmap = 1;
           tbb [1].idCommand = IDM_PAUSE;
           tbb [1].fsState = 0;
           tbb [1].fsStyle = TBSTYLE_BUTTON;
           tbb [1].dwData = 0;
           tbb [1].iString = 0;

           tbb [2].iBitmap = 2;
           tbb [2].idCommand = IDM_STOP;
           tbb [2].fsState = 0;
           tbb [2].fsStyle = TBSTYLE_BUTTON;
           tbb [2].dwData = 0;
           tbb [2].iString = 0;

           g_hWndToolBar = CreateToolbarEx (hWnd,
                                            WS_CHILD | WS_VISIBLE |
                                            TBSTYLE_FLAT,
                                            1,
                                            3,
                                            ((LPCREATESTRUCT) lParam)->hInstance,
                                            1,
                                            tbb,
                                            sizeof (tbb)/sizeof (TBBUTTON),
                                            0,
                                            0,
                                            16,
                                            16,
                                            sizeof (TBBUTTON));
                  
           g_hWndStatusBar = CreateStatusWindow (WS_CHILD | WS_VISIBLE |
                                                 CCS_BOTTOM | SBARS_SIZEGRIP,
                                                 "", hWnd, 2);

           DragAcceptFiles (hWnd, TRUE);
           return 0;
      }

      case WM_DESTROY:
           DragAcceptFiles (hWnd, FALSE);

           KillTimer (hWnd, 1);

           DSCleanup ();
           PostQuitMessage (0);
           return 0;

      case WM_DROPFILES:
      {
           char szFilespec [MAXPATH];
           DragQueryFile ((HDROP) wParam, 0, szFilespec, sizeof (szFilespec));
           SendMessage (hWnd, WM_COMMAND, IDM_FILE_OPEN, (LPARAM) szFilespec);
           DragFinish ((HDROP) wParam);
           return 0;
      }

      case WM_GRAPHNOTIFY:
           if (g_pEvent == NULL)
               return 0;

           // Get all the events

           long evCode;
           LONG_PTR param1, param2;
           HRESULT hr;
           while (SUCCEEDED (g_pEvent->GetEvent (&evCode, &param1, &param2, 0)))
           {
              g_pEvent->FreeEventParams (evCode, param1, param2);

              switch (evCode)
              {
                 case EC_COMPLETE:  // Fall through.
                 case EC_USERABORT: // Fall through.
                 case EC_ERRORABORT:
                      g_pControl->Stop ();
                      g_pPosition->put_CurrentPosition (0);

                      // Disable Pause and Stop buttons.

                      SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PAUSE, 0);
                      SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_STOP, 0);

                      // Enable Play button.
                
                      SendMessage (g_hWndToolBar, TB_SETSTATE, IDM_PLAY,
                                   MAKELONG (TBSTATE_ENABLED, 0));

                      SendMessage (g_hWndStatusBar, SB_SETTEXT, 0, (LPARAM)
                                   (LPSTR) "");
                      return 0;
              }
           } 
           return 0;

      case WM_MOVE:
           if (g_pVidWin != NULL)
               g_pVidWin->NotifyOwnerMessage ((OAHWND) hWnd, iMsg, wParam,
                                              lParam);
           return 0;

      case WM_SIZE:
      {
           SendMessage (g_hWndToolBar, TB_AUTOSIZE, 0, 0);

           int cxParent = LOWORD (lParam);
           int cyParent = HIWORD (lParam);

           RECT rcSB;
           GetWindowRect (g_hWndStatusBar, &rcSB);
           int cy = rcSB.bottom-rcSB.top;
           int cx = cxParent;
           MoveWindow (g_hWndStatusBar, 0, cyParent-cy, cx, cy, TRUE);

           RECT rcTB;
           GetWindowRect (g_hWndToolBar, &rcTB);
           int cyTB = rcTB.bottom-rcTB.top;

           if (g_pVidWin != NULL)
           {
               RECT rc;
               GetClientRect (hWnd, &rc);
               g_pVidWin->SetWindowPosition (0, cyTB, rc.right, rc.bottom-cy-
                                             cyTB);
           }

           aiPartWidths [0] = 3*cxParent/4;
           aiPartWidths [1] = aiPartWidths [0]+cxParent/8-10;
           aiPartWidths [2] = aiPartWidths [1]+cxParent/8-10;

           SendMessage (g_hWndStatusBar, SB_SETPARTS, 3, (LPARAM) (LPINT)
                        aiPartWidths);
           return 0;
      }

      case WM_TIMER:
           if (g_pPosition != NULL)
               UpdateStatusBar ();
           return 0;
   }

   return DefWindowProc (hWnd, iMsg, wParam, lParam);
}

#pragma argsused
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hInstancePrev,
                    LPSTR lpszCmdLine, int iCmdShow)
{
   if (FAILED (CoInitialize (NULL)))
   {
       cerr << "CoInitialize() failure\n";
       return -1;
   }

   WNDCLASSEX wc;
   wc.cbSize = sizeof (WNDCLASSEX);
   wc.style = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc = (WNDPROC) WndProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = 0;
   wc.hInstance = hInstance;
   wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
   wc.hCursor = LoadCursor (NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);
   wc.lpszMenuName = g_szAppName;
   wc.lpszClassName = g_szAppName;
   wc.hIconSm = LoadIcon (NULL, IDI_APPLICATION);

   RegisterClassEx (&wc);

   HWND hWnd = CreateWindowEx (0, g_szAppName, "Media Player #2",
                               WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
                               CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                               CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

   // Keep trying to obtain a timer until one is available. It's unlikely that
   // a timer will not be available the first time SetTimer() is called, but it
   // could happen.

   while (!SetTimer (hWnd, 1, DELAY, NULL))
      if (IDCANCEL == MessageBox (hWnd, "no available timers",
                                  g_szAppName, MB_ICONEXCLAMATION |
                                  MB_RETRYCANCEL))
          return -1;

   ShowWindow (hWnd, iCmdShow);
   UpdateWindow (hWnd);

   MSG msg;
   while (GetMessage (&msg, NULL, 0, 0 ))
   {
      TranslateMessage (&msg);
      DispatchMessage (&msg);
   }

   CoUninitialize ();

   return msg.wParam;
}
